--[[
Only if Elephant "Force timestamp" option is
enabled, returns the time. Otherwise, returns
nil.
]]
local function Prat_CaptureTime()
	if Elephant.db.profile.forcetimestamp then
		return time()
	else
		return nil
	end
end

--[[
Handles messages sent by the WoW engine
as well as the ones sent by Prat.
]]
local function HandleMessage(prat_msg, event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if not Elephant.db.profile.prat and prat_msg then return end
	if not Elephant.db.profile.events[event] then return end
	
	local msg, msg2
	-- Channels
	if event == "CHAT_MSG_CHANNEL" or event == "CHAT_MSG_CHANNEL_NOTICE" then
		local larg9 = string.lower(arg9)
		local cIndex
		for k,v in pairs(Elephant.L['generalchats']) do
			if (larg9 == k) or string.find(larg9, k .. " - ") then
				cIndex = k
				break
			end
		end
		if cIndex == nil then cIndex = larg9 end
		
		if Elephant:IsFiltered(cIndex) then return end
		
		if event == "CHAT_MSG_CHANNEL" then
			if not Elephant.dbpc.char.logs[cIndex].enabled then return end
			
			local msg
			if prat_msg then
				msg = {
					['time'] = Prat_CaptureTime(),
					['prat'] = prat_msg
				}
			else
				msg = {
					['time'] = time(),
					['arg1'] = arg1,
					['arg6'] = arg6,
					['arg9'] = arg9
				}
				if arg2 ~= "" then
					msg['arg2'] = arg2
				end
			end
			Elephant:CaptureNewMessage(msg, cIndex)
		end
		
		if event == "CHAT_MSG_CHANNEL_NOTICE" then
			local larg9 = string.lower(arg9)
			local cIndex
			for k,v in pairs(Elephant.L['generalchats']) do
				if (larg9 == k) or string.find(larg9, k .. " - ") then
					cIndex = k
					break
				end
			end
			if cIndex == nil then cIndex = larg9 end
			
			if arg1 == "YOU_JOINED" then
				Elephant:InitCustomStructure(cIndex, arg9)
				
				if not Elephant.dbpc.char.logs[cIndex].enabled then return end
				
				Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = Elephant.L['customchat']['join'] } , cIndex)
				if Elephant.dbpc.char.currentlogindex == cIndex then
					Elephant:UpdateCurrentLogButtons()
				end
			end
			if arg1 == "YOU_LEFT" then
				if not Elephant.dbpc.char.logs[cIndex] then return end
				if not Elephant.dbpc.char.logs[cIndex].enabled then return end
				
				Elephant:CaptureNewMessage( { ['type'] = "SYSTEM", ['arg1'] = Elephant.L['customchat']['leave'] } , cIndex)
				Elephant:CaptureNewMessage( { ['arg1'] = " " } , cIndex)
				if Elephant.dbpc.char.currentlogindex == cIndex then
					Elephant:UpdateCurrentLogButtons()
					Elephant:ForceCurrentLogDeleteButtonStatus(true)
				end
			end
		end
	-- Not channel messages
	else
		if prat_msg then
			msg = {
				['time'] = Prat_CaptureTime(),
				['prat'] = prat_msg,
				['type'] = Elephant.db.profile.events[event].type
			}
		else
			msg = {
				['time'] = time(),
				['type'] = Elephant.db.profile.events[event].type,
				['arg1'] = arg1
			}
			
			if	event == "CHAT_MSG_BATTLEGROUND" or
				event == "CHAT_MSG_BATTLEGROUND_LEADER" or
				event == "CHAT_MSG_WHISPER" or
				event == "CHAT_MSG_WHISPER_INFORM" or
				event == "CHAT_MSG_MONSTER_WHISPER" or
				event == "CHAT_MSG_RAID" or
				event == "CHAT_MSG_RAID_LEADER" or
				event == "CHAT_MSG_RAID_WARNING" or
				event == "CHAT_MSG_PARTY" or
				event == "CHAT_MSG_PARTY_LEADER" or
				event == "CHAT_MSG_SAY" or
				event == "CHAT_MSG_MONSTER_SAY" or
				event == "CHAT_MSG_YELL" or
				event == "CHAT_MSG_MONSTER_YELL" or
				event == "CHAT_MSG_OFFICER" or
				event == "CHAT_MSG_GUILD" or
				event == "CHAT_MSG_EMOTE" or
				event == "CHAT_MSG_MONSTER_EMOTE" or
				event == "CHAT_MSG_BN_WHISPER" or
				event == "CHAT_MSG_BN_WHISPER_INFORM" or
				event == "CHAT_MSG_ACHIEVEMENT" or
				event == "CHAT_MSG_GUILD_ACHIEVEMENT"
			then
				msg.arg2 = arg2
			end
			
			if event == "CHAT_MSG_WHISPER" then
				msg.arg6 = arg6
			end
			
			if event == "PARTY_LOOT_METHOD_CHANGED" then
				local method, masterloot_party, masterloot_raid = GetLootMethod()
				
				if masterloot_party or masterloot_raid then
					local player
					if UnitInRaid("player") then
						player = GetRaidRosterInfo(masterloot_raid)
					else
						if masterloot_party > 0 then
							player = UnitName("party" .. masterloot_party)
						else
							player = UnitName("player")
						end
					end
					
					msg2 = {
						['time'] = time(),
						['type'] = Elephant.db.profile.events[event].type
					}
					
					-- Name of player may be unknown here, if interface
					-- has just been loaded
					if player == "Unknown" then
						msg2.arg1 = "<" .. Elephant.L['masterlooternameunknown'] .. ">"
					elseif player ~= Elephant.tempConf.masterlooter then
						Elephant.tempConf.masterlooter = player
						
						msg2.arg1 =  format(Elephant.L['masterlooterchanged'], player)
					end
				else
					Elephant.tempConf.masterlooter = nil
				end
				
				if method ~= Elephant.tempConf.lootmethod then
					Elephant.tempConf.lootmethod = method
					msg.arg1 = Elephant.L['lootmethod'][method]
				else
					msg = msg2
					msg2 = nil
				end
			end
		end

		local k
		for k in pairs(Elephant.db.profile.events[event].channels) do
			if Elephant.db.profile.events[event].channels[k] ~= 0 and Elephant.dbpc.char.logs[k].enabled then
				Elephant:CaptureNewMessage(msg, k)
				if msg2 ~= nil then
					Elephant:CaptureNewMessage(msg2, k)
				end
			end
		end
	end
end

--[[
Unregisters all events from Elephant, and then
registers back either:
- Only one Prat event if the "Prat formatting"
  option is enabled.
- All WoW events defined in Core.lua

This method displays a message if the "Prat
formatting" option is checked but Prat is not
loaded.
]]
function Elephant:RegisterEventsRefresh()
	Elephant:UnregisterAllEvents()
	
	if Prat	and Elephant.db.profile.prat then
		Prat.RegisterChatEvent(Elephant, Prat.Events.POST_ADDMESSAGE)
	else
		if not Prat and Elephant.db.profile.prat then
			Elephant:Print("|cffff0000" .. Elephant.L['noprat'] .. "|r")
		end
	
		local event
		for event,v in pairs(Elephant.db.profile.events) do
			Elephant:RegisterEvent(event, HandleMessage, nil)
		end
	end
end

--[[
Method pre-handling messages sent by Prat before
sending them to HandleMessage()
]]
-- Cannot be local
function Elephant:Prat_PostAddMessage(_, message, _, event, text)
	HandleMessage(text, event, message.ORG.MESSAGE, _, _, _, _, _, _, _, message.ORG.CHANNEL)
end