--[[
Registers the options of the addon and sets up
any configuration window or slash command
giving access to them.
]]
function Elephant:SetupOptions()
	-- First, registering options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Elephant", {
		type = 'group',
		args = {
			togglebutton = {
				type = 'toggle',
				name = Elephant.L['showbutton'],
				desc = Elephant.L['showbutton_desc'],
				get = function()
					return Elephant.db.profile.button
				end,
				set = Elephant.ToggleButton
			},
			toggleicon = {
				type = 'toggle',
				name = Elephant.L['minimapicon'],
				desc = Elephant.L['minimapicon_desc'],
				get = function()
					return not Elephant.db.profile.minimap.hide
				end,
				set = Elephant.ToggleLDBIcon,
				hidden = function()
					return not Elephant:IsLDBIconAvailable()
				end
			},
			prat = {
				type = 'toggle',
				name = Elephant.L['prat'],
				desc = Elephant.L['prat2_desc'] .. "\n\n|c" ..
					Elephant:MakeTextHexColor(0.2, 1.0, 0.2) .. Elephant.L['prat2_desc2'] .. "|r",
				get = function()
					return Elephant.db.profile.prat
				end,
				set = function(_, v)
					Elephant.db.profile.prat = v
					Elephant:RegisterEventsRefresh()
				end
			},
			forcetimestamp = {
				type = 'toggle',
				name = Elephant.L['forcetimestamp'],
				desc = Elephant.L['forcetimestamp2_desc'] .. "\n\n|c" ..
					Elephant:MakeTextHexColor(0.2, 1.0, 0.2) .. Elephant.L['forcetimestamp2_desc2'] .. "|r",
				get = function()
					return Elephant.db.profile.forcetimestamp
				end,
				set = function(_, v)
					Elephant.db.profile.forcetimestamp = v
				end,
				disabled = function()
					return not Elephant.db.profile.prat
				end,
			},
			filters = {
				type = 'group',
				name = Elephant.L['Filters'],
				desc = Elephant.L['Filters_desc'],
				args = {
					add = {
						type = 'input',
						name = NEW,
						desc = Elephant.L['newfilter_desc'],
						get = false,
						set = function(_, input)
							if string.match(input, Elephant.L['filterregex']) == nil then
								Elephant:Print(format(Elephant.L['filtererror'], input))
							else
								Elephant:AddFilter(input)
							end
						end,
						usage = Elephant.L['filterusage'],
					},
					delete = {
						type = 'select',
						name = DELETE,
						desc = Elephant.L['deletefilter_desc'],
						set = function(_,filterindex)
							Elephant:DeleteFilter(filterindex)
						end,
						values = function()
							return 	Elephant.db.profile.filters
						end,
						hidden = function()
							return (not Elephant.db.profile.filters) or
								(#Elephant.db.profile.filters == 0)
						end
					}
				},
			},
			log = {
				type = 'group',
				name = Elephant.L['logs'],
				desc = Elephant.L['logs_desc'],
				args = {
					clearall = {
						type = 'execute',
						name = Elephant.L['clearallhelp'],
						desc = Elephant.L['clearallhelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_CLEARALL")
						end,
					},
					default = {
						type = 'toggle',
						name = Elephant.L['enableddefault'],
						desc = Elephant.L['enableddefault_desc'],
						get = function()
							return Elephant.db.profile.defaultlog
						end,
						set = function(_, isEnabled)
							Elephant.db.profile.defaultlog = isEnabled
						end,
					},
					max = {
						type = 'range',
						name = Elephant.L['maxlogwords'],
						desc = Elephant.L['maxlogwords_desc'],
						min = Elephant.defaultConf.minlogsize,
						max = Elephant.defaultConf.maxlogsize,
						step = 1,
						get = function()
							return Elephant.db.profile.maxlog
						end,
						set = function(_, nb)
							Elephant:ChangeMaxLog(nb)
						end,
					},
					files = {
						type = 'group',
						name = Elephant.L['files'],
						desc = Elephant.L['files_desc'],
						args = {
							activate = {
								type = 'toggle',
								name = Elephant.L['activate'],
								desc = Elephant.L['activate_desc'],
								get = function()
									return Elephant.db.profile.activate_log
								end,
								set = function(_, nv)
									Elephant.db.profile.activate_log = nv
									Elephant:ChatLogEnable(Elephant.db.profile.chatlog)
									Elephant:CombatLogEnable(Elephant.db.profile.combatlog)
								end,
							},
							chat = {
								type = 'toggle',
								name = Elephant.L['chatlog'],
								desc = Elephant.L['chatlog2_desc'] .. "\n\n|c" ..
									Elephant:MakeTextHexColor(0.2, 1.0, 0.2) .. Elephant.L['chatlog2_desc2'] .. "|r",
								get = function()
									return Elephant.db.profile.chatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.chatlog = isEnabled
									Elephant:ChatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
							combat = {
								type = 'toggle',
								name = Elephant.L['combatlog'],
								desc = Elephant.L['combatlog2_desc'] .. "\n\n|c" ..
									Elephant:MakeTextHexColor(0.2, 1.0, 0.2) .. Elephant.L['combatlog2_desc2'] .. "|r",
								get = function()
									return Elephant.db.profile.combatlog
								end,
								set = function(_, isEnabled)
									Elephant.db.profile.combatlog = isEnabled
									Elephant:CombatLogEnable(isEnabled)
								end,
								hidden = function()
									return not Elephant.db.profile.activate_log
								end,
							},
						},
					},
				},
			},
			reset = {
				type = 'group',
				name = Elephant.L['reset'],
				desc = Elephant.L['reset_desc'],
				args = {
					all = {
						type = 'execute',
						name = Elephant.L['resethelp'],
						desc = Elephant.L['resethelp_desc'],
						func = function()
							StaticPopup_Show("ELEPHANT_RESET")
						end,
					},
					position = {
						type = 'execute',
						name = Elephant.L['resetloc'],
						desc = Elephant.L['resetloc_desc'],
						func = function()
							Elephant:ResetPosition()
						end,
					},
				},
			},
		}
	})
	
	-- Adding options to blizzard frame
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Elephant")

	-- Adding slash command to access options
	Elephant:RegisterChatCommand("elephant", function()
		InterfaceOptionsFrame_OpenToCategory("Elephant")
	end)
end